@@ -130,43 +130,56 @@ static void open_unix(void)
 }
 
 
-static int got_unix(void *buf,int len,void *user)
+void notify(const UN_CTX *ctx,uint32_t ip,const ENTRY *entry)
 {
-    struct atmarp_req *req = buf;
-    int *reply = buf;
+    struct atmarp_req reply;
 
-    if (len != sizeof(*req)) {
-	diag(COMPONENT,DIAG_ERROR,"bad unix read: %d != %d",len,sizeof(*req));
-	return 0;
-    }
-    switch (req->type) {
-	case art_create:
-	    *reply = ioctl(kernel,SIOCMKCLIP,req->itf);
-	    if (*reply >= 0) itf_create(*reply);
-	    break;
-	case art_qos:
-	case art_set:
-	case art_delete:
-	    *reply = arp_ioctl(req);
-	    break;
-	case art_table:
-	    *reply = table_update();
-	    break;
-	default:
-	    diag(COMPONENT,DIAG_ERROR,"invalid request msg type 0x%x",
-	      req->type);
-	    *reply = -EINVAL;
-    }
-    return sizeof(int);
+    memset(&reply,0,sizeof(reply));
+    reply.type = art_query;
+    reply.ip = ip;
+    if (entry) reply.addr = *entry->addr;
+    if (un_send(ctx,&reply,sizeof(reply)) < 0)
+        diag(COMPONENT,DIAG_WARN,"notify: %s",strerror(errno));
 }
 
 
 static void recv_unix(void)
 {
+    UN_CTX ctx;
     struct atmarp_req req;
+    int len,reply;
 
-    if (un_reply(unix_sock,&req,sizeof(req),got_unix,NULL) < 0)
-	diag(COMPONENT,DIAG_ERROR,"un_reply: %s",strerror(errno));
+    len = un_recv(&ctx,unix_sock,&req,sizeof(req));
+    if (len < 0) {
+	diag(COMPONENT,DIAG_ERROR,"recv_unix: %s",strerror(errno));
+	return;
+    }
+    if (len != sizeof(req)) {
+	diag(COMPONENT,DIAG_ERROR,"bad unix read: %d != %d",len,sizeof(req));
+	return;
+    }
+    switch (req.type) {
+	case art_create:
+	    reply = ioctl(kernel,SIOCMKCLIP,req.itf);
+	    if (reply >= 0) itf_create(reply);
+	    break;
+	case art_qos:
+	case art_set:
+	case art_delete:
+	    reply = arp_ioctl(&req);
+	    break;
+	case art_table:
+	    reply = table_update();
+	    break;
+	case art_query:
+	    query_ip(&ctx,req.ip);
+	    return;
+	default:
+	    diag(COMPONENT,DIAG_ERROR,"invalid request msg type 0x%x",req.type);
+	    reply = -EINVAL;
+    }
+    if (un_send(&ctx,&reply,sizeof(reply)) < 0)
+	diag(COMPONENT,DIAG_ERROR,"un_send: %s",strerror(errno));
 }
 
 
@@ -221,13 +234,45 @@ static void recv_vcc(VCC *vcc)
 }
 
 
+static void drain_vcc(VCC *vcc)
+{
+    unsigned char buffer[MAX_BUFFER];
+    char line[80]; /* actually, it's only 7+16*3+1 */
+    int size;
+    int i;
+
+    size = read(vcc->fd,buffer,MAX_BUFFER);
+    if (!size) {
+	disconnect_vcc(vcc);
+	return;
+    }
+    if (size < 0) {
+	diag(COMPONENT,DIAG_ERROR,"read vcc: %s",strerror(errno));
+	disconnect_vcc(vcc);
+	return;
+    }
+    diag(COMPONENT,DIAG_WARN,"drain_vcc: unexpected message on "
+      "unidirectional (RSVP?) VCC %p:",vcc);
+    for (i = 0; i < size; i++) {
+	if (!(i & 15)) {
+	    if (i) diag(COMPONENT,DIAG_WARN,"%s",line);
+	    sprintf(line,"  %04x:",i);
+	    *line = 0;
+	}
+	sprintf(strchr(line,0)," %02x",buffer[i]);
+    }
+    diag(COMPONENT,DIAG_WARN,"%s",line);
+}
+
+
 static void accept_new(void)
 {
     char buffer[MAX_ATM_ADDR_LEN+1];
     struct sockaddr_atmsvc addr;
+    struct atm_qos qos;
     ENTRY *entry;
     VCC *vcc;
-    int fd,len,error;
+    int fd,len,size,error;
 
     len = sizeof(addr);
     if ((fd = accept(incoming,(struct sockaddr *) &addr,&len)) < 0) {
@@ -240,7 +285,17 @@ static void accept_new(void)
 	}
 	return;
     }
-    if (ioctl(fd,ATMARP_MKIP,CLIP_DEFAULT_IDLETIMER) < 0) {
+    /* the following code probably belongs to arp.c ... */
+    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,pretty) <
+      0) strcpy(buffer,"<atm2text error>");
+    diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
+    size = sizeof(qos);
+    if (getsockopt(fd,SOL_ATM,SO_ATMQOS,&qos,&size) < 0)
+	diag(COMPONENT,DIAG_FATAL,"getsockopt SO_ATMQOS: %s",strerror(errno));
+    if (size != sizeof(qos))
+	diag(COMPONENT,DIAG_FATAL,"SO_ATMQOS: size %d != %d",size,sizeof(qos));
+    if (ioctl(fd,ATMARP_MKIP,qos.txtp.traffic_class == ATM_NONE ? 0 :
+      CLIP_DEFAULT_IDLETIMER) < 0) {
         diag(COMPONENT,DIAG_ERROR,"ioctl ATMARP_MKIP: %s",strerror(errno));
         (void) do_close(fd);
         return;
@@ -249,10 +304,12 @@ static void accept_new(void)
     vcc->active = 0;
     vcc->connecting = 0;
     vcc->fd = fd;
-    /* the following code probably belongs to arp.c ... */
-    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,pretty) <
-      0) strcpy(buffer,"<atm2text error>");
-    diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
+    if (qos.txtp.traffic_class == ATM_NONE) {
+	vcc->entry = NULL;
+	incoming_unidirectional(vcc);
+	Q_INSERT_HEAD(unidirectional_vccs,vcc);
+	return;
+    }
     if (merge) {
 	ITF *itf;
 
@@ -406,6 +463,10 @@ void poll_loop(void)
 	    FD_SET(entry->vccs->fd,&rset);
 	    if (entry->vccs->fd >= fds) fds = entry->vccs->fd+1;
 	}
+	for (vcc = unidirectional_vccs; vcc; vcc = vcc->next) {
+	    FD_SET(vcc->fd,&rset);
+	    if (vcc->fd >= fds) fds = vcc->fd+1;
+	}
 	ret = select(fds,&rset,&cset,NULL,next_timer());
 /*
  * Now here's something strange: < 0.32 needed the exception mask to be NULL
@@ -453,6 +514,10 @@ void poll_loop(void)
 		next_entry = entry->next;
 		if (FD_ISSET(entry->vccs->fd,&rset)) recv_vcc(entry->vccs);
 	    }
+	    for (vcc = unidirectional_vccs; vcc; vcc = next_vcc) {
+		next_vcc = vcc->next;
+		if (FD_ISSET(vcc->fd,&rset)) drain_vcc(vcc);
+	    }
 	    expire_timers();
 	      /* expire timers after handling messages to make sure we don't
 		 time out unnecessarily because of scheduling delays */
