@@ -49,7 +49,7 @@ static int atm_parse_class_opt(struct qdisc_util *qu, int argc, char **argv,
 	struct atm_qos qos;
 	struct atm_sap sap;
 	unsigned char hdr[MAX_HDR_LEN];
-	__u32 excess;
+	__u32 excess = 0;
 	struct rtattr *tail;
 	int hdr_len = -1;
 	int s;
@@ -151,7 +151,7 @@ static int atm_parse_class_opt(struct qdisc_util *qu, int argc, char **argv,
 	tail = (struct rtattr *) (((void *) n)+NLMSG_ALIGN(n->nlmsg_len));
 	addattr_l(n,1024,TCA_OPTIONS,NULL,0);
 	addattr_l(n,1024,TCA_ATM_FD,&s,sizeof(s));
-	addattr_l(n,1024,TCA_ATM_EXCESS,&excess,sizeof(excess));
+	if (excess) addattr_l(n,1024,TCA_ATM_EXCESS,&excess,sizeof(excess));
 	if (hdr_len != -1) addattr_l(n,1024,TCA_ATM_HDR,hdr,hdr_len);
 	tail->rta_len = (((void *) n)+NLMSG_ALIGN(n->nlmsg_len))-(void *) tail;
 	return 0;
@@ -175,7 +175,7 @@ static int atm_print_opt(struct qdisc_util *qu, FILE *f, struct rtattr *opt)
 			if (atm2text(buffer,MAX_ATM_ADDR_LEN,
 			    RTA_DATA(tb[TCA_ATM_ADDR]),A2T_PRETTY | A2T_NAME) <
 			    0) fprintf(stderr,"atm2text error\n");
-			fprintf(f,"pvc %s",buffer);
+			fprintf(f,"pvc %s ",buffer);
 		}
 	if (tb[TCA_ATM_HDR]) {
 		int i;
@@ -185,6 +185,7 @@ static int atm_print_opt(struct qdisc_util *qu, FILE *f, struct rtattr *opt)
 			fprintf(f,"%c%02x",i ? '.' : ' ',
 			    ((unsigned char *) RTA_DATA(tb[TCA_ATM_HDR]))[i]);
 		if (!i) fprintf(f," .");
+		fprintf(f," ");
 	}
 	if (tb[TCA_ATM_EXCESS]) {
 		__u32 excess;
