@@ -56,7 +56,7 @@ void ilmi_loop(int fd, int itf);
 
 void usage(char *name)
 {
-  fprintf(stderr, "usage: %s [-b] [-d] [-v] [-l logfile] [-x] [-q qos] "
+  fprintf(stderr, "usage: %s [-b] [-d] [-v] [-l logfile] [-x] [-q qos]\n"
     "%13s[ -i local_ip ] "
 #ifdef DYNAMIC_UNI
     "[ -u uni ] "
@@ -275,7 +275,7 @@ int main(int argc, char *argv[])
   qos = NULL;
   while((opt = getopt(argc, argv, "bdhi:l:q:xv"
 #ifdef DYNAMIC_UNI
-    "u"
+    "u:"
 #endif
     )) != EOF)
     switch (opt)
@@ -315,6 +315,7 @@ int main(int argc, char *argv[])
 	else if (!strcmp(optarg,"4.0") || !strcmp(optarg,"40"))
 	  atmfAtmLayerUniVersionValue = 4; // version4point0(4)
 	else usage(argv[0]);
+	break;
 #endif
       case 'h':
       default:
