@@ -882,9 +882,9 @@ conn_check_incoming(fd_set *fds)
 int
 conn_create_listensocket(HANDLE sap_handle, HANDLE conn_context,
 			 unsigned short blli_codepoint, const char *qos_spec,
-			 HANDLE *p_conn_handle)
+			 LAN_MTU mtu)
 {
-  int fd, ret, len;
+  int fd, ret, len, sdu;
   Conn_t *conn;
   CONN_INFO *conn_info;
   struct atm_sap atmsap;
@@ -906,11 +906,15 @@ conn_create_listensocket(HANDLE sap_handle, HANDLE conn_context,
 
   switch (blli_codepoint) {
   case BLLI_CONTROL:
-  case BLLI_DIRECT_802_3:
-  case BLLI_BUS_802_3:
     conn_info->conqos.rxtp.max_sdu = 1516;
     conn_info->conqos.txtp.max_sdu = 1516;
     break;
+  case BLLI_DIRECT_802_3:
+  case BLLI_BUS_802_3:
+    sdu = maxframe2maxsdu(mtu);
+    conn_info->conqos.rxtp.max_sdu = sdu;
+    conn_info->conqos.txtp.max_sdu = sdu;
+    break;
   case BLLI_DIRECT_802_5:
   case BLLI_BUS_802_5:
     conn_info->conqos.rxtp.max_sdu = 4544;
@@ -984,8 +988,6 @@ conn_create_listensocket(HANDLE sap_handle, HANDLE conn_context,
   EVENT(EM_EVENT,("Listen socket created blli:%2.2x %2.2x\n",
 		  conn_info->blli.l3.tr9577.snap[3],
 		  conn_info->blli.l3.tr9577.snap[4]));
-  if (p_conn_handle)
-    *p_conn_handle = conn;
   return fd;
 }
 
@@ -1063,7 +1065,35 @@ conn_set_kernel_socket(int fd)
   return fd;
 }
 
+int
+maxframe2maxsdu(LAN_MTU mtu)
+{
 
+    int sdu;
+
+    switch (mtu) {
+    case MTU_1516:
+        sdu = 1516;
+        break;
+    case MTU_1580:  /* LANE2: MTU can be 1580 too (IEEE 802.1p/Q) */
+        sdu = 1580;
+        break;
+    case MTU_4544:
+        sdu = 4544;
+        break;
+    case MTU_9234:
+        sdu = 9234;
+        break;
+    case MTU_18190:
+        sdu = 18190;
+        break;
+    default:
+        sdu = 1516;
+        break;
+    }
+
+    return sdu;
+}
 /*
  *
  * $Log: conn.c,v $
