@@ -31,7 +31,7 @@ extern FILE *yyin;
 
 int debug = 0;
 int pretty = A2T_PRETTY | A2T_NAME | A2T_LOCAL;
-const char *dump_dir = NULL;
+const char *dump_dir = DEFAULT_DUMP_DIR;
 
 
 /* A little hack until we have full support for multiple signaling entities */
@@ -328,7 +328,6 @@ int main(int argc,char **argv)
 
     set_application("atmsigd");
     config_file = CONFIG_FILE;
-    dump_dir = NULL;
     background = 0;
     memset(&_entity.signaling_pvc,0,sizeof(_entity.signaling_pvc));
     /* 1st pass to get the -c option */
@@ -364,7 +363,6 @@ int main(int argc,char **argv)
 		break;
 	    case 'D':
 		dump_dir = optarg;
-		if (!trace_size) trace_size = DEFAULT_TRACE_SIZE;
 		break;
 	    case 'l':
 		set_logfile(optarg);
@@ -430,6 +428,7 @@ int main(int argc,char **argv)
 	}
     }
     if (optind != argc) usage(argv[0]);
+    if (!trace_size) dump_dir = NULL;
     if (dump_dir)
 	if (chdir(dump_dir) < 0)
 	    diag(COMPONENT,DIAG_ERROR,"chdir %s: %s",dump_dir,strerror(errno));
