@@ -91,14 +91,9 @@ static int send_setup(SOCKET *sock)
 	error = sap_encode(&dsc,&sock->remote,&sock->sap,&sock->qos,
 	  sock->sig->uni);
     q_assign(&dsc,QF_bearer_class,16); /* BCOB-X */
-#if defined(UNI30) || defined(ALLOW_UNI30) || defined(DYNAMIC_UNI)
-    if (!error && sock->sig->uni & S_UNI30)
-	q_assign(&dsc,QF_trans_cap,ATM_TC_VBR_NRT_R00);
-	  /* force presence - UNI 3.0 wants this */
-#endif
     q_assign(&dsc,QF_upcc,0); /* p2p */
 #if !defined(UNI30) || defined(DYNAMIC_UNI)
-    if (!errror && sock->sig->uni != S_UNI30)
+    if (!error && sock->sig->uni != S_UNI30)
 	q_assign(&dsc,QF_qos_cs,Q2931_CS_ITU);
 #endif
     q_assign(&dsc,QF_qos_fw,0); /* QOS 0 */
