@@ -421,7 +421,7 @@ void add_route(SIG_ENTITY *sig,struct sockaddr_atmsvc *addr,int len)
 }
 
 
-SIG_ENTITY *route_call(struct sockaddr_atmsvc *addr)
+SIG_ENTITY *route_remote(struct sockaddr_atmsvc *addr)
 {
     ROUTE *best,*route;
     int best_len;
@@ -451,3 +451,16 @@ SIG_ENTITY *get_sig_entity(int itf)
     /* should also check for vpcis @@@ */
     return NULL;
 }
+
+
+SIG_ENTITY *route_local(struct sockaddr_atmsvc *addr)
+{
+    LOCAL_ADDR *walk;
+
+    for (walk = local_addr; walk->state != ls_unused; walk++)
+	if (walk->state == ls_same && atm_equal((struct sockaddr *) &walk->addr,
+	  (struct sockaddr *) addr,(ATM_ESA_LEN-1)*8,AXE_WILDCARD))
+	    break;
+    if (walk->state == ls_unused) return NULL;
+    return get_sig_entity(walk->itf);
+}
