@@ -103,7 +103,7 @@ static void setup_call(SIG_ENTITY *sig,unsigned long call_ref)
     this->call_state = cs_in_proc;
     this->call_ref = call_ref;
     if (q_present(&in_dsc,QF_ep_ref))
-	this->ep_ref = q_fetch(&in_dsc,QF_ep_ref);
+	this->ep_ref = q_fetch(&in_dsc,QF_ep_ref) ^ 0x8000;
 #ifdef CISCO
     else
 #endif
@@ -749,14 +749,14 @@ static void process_uni(SIG_ENTITY *sig,void *msg)
 	return;
     }
     if (mid == ATM_MSG_STATUS_ENQ) {
-	send_status(curr->sig,curr,call_ref,ATM_CV_RESP_STAT_ENQ);
+	send_status(sig,curr,call_ref,ATM_CV_RESP_STAT_ENQ);
 	return;
     }
     if (curr && q_present(&in_dsc,QF_ep_ref) && mid != ATM_MSG_ADD_PARTY &&
       mid != ATM_MSG_DROP_PARTY_ACK)
-	if (curr->ep_ref != q_fetch(&in_dsc,QF_ep_ref)) {
-	    send_drop_party_ack(sig,call_ref,q_fetch(&in_dsc,QF_ep_ref),
-	      ATM_CV_INV_EPR);
+	if (curr->ep_ref != q_fetch(&in_dsc,QF_ep_ref) ^ 0x8000) {
+	    send_drop_party_ack(sig,call_ref,
+	      q_fetch(&in_dsc,QF_ep_ref) ^ 0x8000,ATM_CV_INV_EPR);
 	    return;
 	}
     if (!curr || curr->call_state == cs_null) {
